using BuaaLocationServer.Dto;
using BuaaLocationServer.Middlewares.Db;
using BuaaLocationServer.Middlewares.Jwts;
using BuaaLocationServer.Models;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Connections;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Caching.Memory;
using Microsoft.Extensions.Options;
using Microsoft.IdentityModel.JsonWebTokens;
using System.ComponentModel.DataAnnotations;
using System.Security.Claims;

namespace BuaaLocationServer.Controllers
{
    /// <summary>
    /// Ȩ
    /// </summary>
    [ApiController]
    [Authorize]
    [Route("api/[controller]")]
    public class AuthenticationController : BaseController
    {
        private readonly BuaaDbContext _context;
        private readonly ILogger<AuthenticationController> _logger;
        private readonly IMemoryCache _memoryCache;
        private readonly IJwtService _jwtService;
        private readonly IOptionsMonitor<JwtOptions> _jwtOptions;

        public AuthenticationController(
            ILogger<AuthenticationController> logger,
            BuaaDbContext context,
            IMemoryCache memoryCache,
            IJwtService jwtService,
            IOptionsMonitor<JwtOptions> jwtOptions)
        {
            _logger = logger;
            _context = context;
            _memoryCache = memoryCache;
            _jwtService = jwtService;
            _jwtOptions = jwtOptions;
        }

        /// <summary>
        /// ¼
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("[action]")]
        [AllowAnonymous]
        public async Task<ActionResult<UserDto>> SignIn([FromBody] UserDto dto)
        {
            var user = await _context.Users.FirstOrDefaultAsync(e => e.UserName == dto.UserName);

            if (user == null || !BCrypt.Net.BCrypt.Verify(dto.Password, user.Password))
                return Problem("û");


            var claims = new List<Claim>
            {
                new Claim(JwtClaimTypes.Subject, user.UserName),
                new Claim(JwtClaimTypes.Role, user.RoleType.ToString())
            };
            var identity = new ClaimsIdentity(claims);
            var token = _jwtService.Create(identity);
            var refreshToken = Guid.NewGuid().ToString("N");
            var expireTime = DateTimeOffset.Now.AddDays(1);
            _memoryCache.Set(refreshToken, dto.UserName, expireTime);
            return Ok(new { token, expireTime, dto.UserName, refreshToken });
        }

        /// <summary>
        /// ˢơ
        /// </summary>
        [AllowAnonymous]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status400BadRequest)]
        [ProducesResponseType(StatusCodes.Status401Unauthorized)]
        [HttpPost("[action]")]
        public ActionResult<object> RefreshToken([FromBody] RefreshTokenDto dto)
        {
#if RELEASE
            //У黺Ƿиˢ
            if (!_memoryCache.TryGetValue<string>(dto.RefreshToken, out _))
                return Unauthorized();
#endif

            //УǷЧ
            if (!_jwtService.Validate(dto.Token, out var principal))
                return Unauthorized();

            var jwtOptions = _jwtOptions.CurrentValue;
            if (principal.Identity is not ClaimsIdentity identity)
                return Unauthorized();

            var newToken = _jwtService.Create(identity);

            var result = new
            {
                Token = newToken,
                RefreshToken = dto.RefreshToken,
                Expires = jwtOptions.Expires
            };

            return Ok(result);
        }

        /// <summary>
        /// ȡǰ¼û
        /// </summary>
        /// <returns></returns>
        [HttpGet("[action]")]
        public async Task<ActionResult<User>> Profile()
        {
            var userName = this.GetCurrentUserName();
            var user = await _context.Users.FirstOrDefaultAsync(e => e.UserName == userName);
            return Ok(user);
        }

    }

    /// <summary>
    /// ˢϢ
    /// </summary>
    public class RefreshTokenDto
    {
        /// <summary>
        /// JWTơ
        /// </summary>
        [Required(AllowEmptyStrings = false)]
        public string Token { get; set; } = null!;

        /// <summary>
        /// ˢơ
        /// </summary>
        [Required(AllowEmptyStrings = false)]
        public string RefreshToken { get; set; } = null!;
    }
}